<?php

namespace Drupal\Tests\instana_eum\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\DocumentElement;

/**
 * Base configuration for Functional Javascript tests.
 */
abstract class FunctionalJsBase extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'instana_eum',
  ];

  /**
   * A test administrator.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * A regular authenticated user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create users:
    $this->user = $this->drupalCreateUser();

    $this->adminUser = $this->drupalCreateUser();
    $this->adminUser->addRole($this->createAdminRole('administrator', 'administrator'));
    $this->adminUser->set('name', 'CR7');
    $this->adminUser->save();
  }

  /**
   * Looks for the specified text and returns TRUE when it is unavailable.
   *
   * Core JSWebAssert has a function waitForText() but there is no equivalent to
   * wait until text is hidden, as there is for some other page elements.
   * Therefore define that function here, based on waitForText() in
   * core/tests/Drupal/FunctionalJavascriptTests/JSWebAssert.php.
   *
   * @param string $text
   *   The text to wait for.
   * @param int $timeout
   *   (Optional) Timeout in milliseconds, defaults to 10000.
   *
   * @return bool
   *   TRUE if not found, FALSE if found.
   */
  public function waitForNoText($text, $timeout = 10000) {
    $page = $this->getSession()->getPage();
    return (bool) $page->waitFor($timeout / 1000, function (DocumentElement $page) use ($text) {
      $actual = preg_replace('/\\s+/u', ' ', $page->getText());
      // Negative look-ahead on the text that should be hidden.
      $regex = '/^((?!' . preg_quote($text, '/') . ').)*$/ui';
      return (bool) preg_match($regex, $actual);
    });
  }

}
